"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var assertions_1 = require("./assertions");
var ts = __importStar(require("typescript"));
function parseSourceFile(src, checker, generator) {
    var symbols = [];
    traverse(src);
    function traverse(node) {
        switch (node.kind) {
            // case ts.SyntaxKind.TypeAliasDeclaration: {
            //     const n = node as ts.TypeAliasDeclaration;
            //     const runtimeInfo = getRuntimeName(n, src);
            //     if (!runtimeInfo) {
            //         break;
            //     }
            //     const symbol = assertExists(checker.getSymbolAtLocation(n.name));
            //     console.log(`Found runtime interface: ${runtimeInfo.original} => ${runtimeInfo.alias}`);
            //     const r = assertExists(
            //         generator.getSymbols(runtimeInfo.original).find((item) => item.symbol === symbol)
            //     );
            //     const srcPath: string = (src as any).path || src.fileName;
            //     symbols.push({
            //         refName: r.name,
            //         alias: runtimeInfo.alias,
            //         srcPath
            //     });
            //     break;
            // }
            case ts.SyntaxKind.TypeAliasDeclaration:
            case ts.SyntaxKind.InterfaceDeclaration: {
                var n = node;
                var runtimeInfo = getRuntimeName(n, src);
                if (!runtimeInfo) {
                    break;
                }
                var symbol_1 = assertions_1.assertExists(checker.getSymbolAtLocation(n.name), 'A node must have a name');
                // console.log(symbol);
                console.log("Found runtime interface: " + runtimeInfo.original + " => " + runtimeInfo.alias);
                var r = assertions_1.assertExists(generator.getSymbols(runtimeInfo.original).find(function (item) { return item.symbol === symbol_1; }), 'A found symbol must exist');
                var srcPath = src.path || src.fileName;
                symbols.push({
                    refName: r.name,
                    alias: runtimeInfo.alias,
                    srcPath: srcPath
                });
                break;
            }
        }
        ts.forEachChild(node, traverse);
    }
    return symbols;
}
exports.parseSourceFile = parseSourceFile;
// TODO: Figure out why I can't reference this directly lol?
var RUNTIME_NAME = /^\/\/ @runtime( +(.*))?$/;
// TODO: Make the '@runtime' keyword configurable
function getRuntimeName(node, src) {
    var runtimeComment = getComments(node, src).find(function (comment) { return comment.startsWith('// @runtime'); });
    if (!runtimeComment) {
        return;
    }
    // TODO: Use RUNTIME_NAME from above
    var m = assertions_1.assertExists(runtimeComment.match(/^\/\/ @runtime( +(.*))?$/), 'We know that @runtime exists because the line starts with it');
    var original = node.name.text;
    if (m[2]) {
        return {
            alias: m[2],
            original: original
        };
    }
    return {
        original: original,
        alias: original
    };
}
function getComments(node, src) {
    return src.text
        .substring(node.getFullStart(), node.getStart())
        .split('\n')
        .map(function (s) { return s.trim(); })
        .filter(function (s) { return s.length; });
}
