"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var tsProgram_1 = require("./tsProgram");
var assertions_1 = require("./assertions");
var tjs = __importStar(require("typescript-json-schema"));
var fs = __importStar(require("fs"));
var parseTypes_1 = require("./parseTypes");
var config_1 = require("./config");
var path = __importStar(require("path"));
var prettier_1 = __importDefault(require("prettier"));
function processProject(options) {
    var program = tsProgram_1.getProgramFromConfig(options.projectPath);
    var sourceFiles = program.getSourceFiles();
    var generator = assertions_1.assertExists(tjs.buildGenerator(program, {
        aliasRef: true,
        required: true
    }), 'Expected typescript generator to be built');
    var checker = program.getTypeChecker();
    var symbols = sourceFiles.reduce(function (acc, src) { return acc.concat(parseTypes_1.parseSourceFile(src, checker, generator)); }, []);
    var schema = generator.getSchemaForSymbols(symbols.map(function (ref) { return ref.refName; }));
    fs.writeFileSync(options.schemaPath, JSON.stringify(schema));
    fs.writeFileSync(options.typeDefinitionsPath, template(options.typeDefinitionsPath, symbols));
}
exports.processProject = processProject;
// tslint:disable-next-line
var truePath = require('true-case-path');
function template(outputPath, aliasData, aliased) {
    if (aliased === void 0) { aliased = ''; }
    var imports = aliasData.map(function (data) {
        var from = truePath(path.normalize(path.dirname(path.resolve(outputPath))).toLowerCase());
        var to = truePath(path.normalize(data.srcPath).toLowerCase());
        var relPath = ("./" + path.relative(from, to)).slice(0, -3).replace(path.sep, '/');
        return "import { " + data.refName + " } from '" + relPath + "';";
    });
    var overrides = aliasData.map(function (data) {
        return "function validate(interfaceName: '" + data.alias + "', obj: any): " + data.refName + ";";
    });
    return ('// prettier-ignore-start\n' +
        prettier_1.default.format("// tslint:disable\n/*\n * This file was automatically generated by something else. Probably shouldn't touch it.\n */\nimport * as _unused from '" + config_1.MODULE_NAME + "';\n" + imports.join('\n') + "\n\ndeclare namespace Validate {\n    " + overrides.join('\n') + "\n    function validate(interfaceName: string, obj: any): never;\n}\n\ndeclare module '" + config_1.MODULE_NAME + "' {\n    // @ts-ignore\n    " + (aliased.length === 0
            ? 'export function validator(schemaPath: string): typeof Validate.validate;'
            : "export function validator(schemaPath: string, schemaName: '" + aliased + "'): typeof Validate.validate;") + "\n\n}\n", { filepath: outputPath }));
}
